from gluesql import Glue, MemoryStorage


def test_payload():
    db = Glue(MemoryStorage())

    assert (
        db.query(
            """
    CREATE TABLE Foo (id INTEGER);
    CREATE TABLE Bar;
    """
        )
        == [{"type": "CREATE TABLE"}, {"type": "CREATE TABLE"}]
    )

    assert (
        db.query(
            """
    INSERT INTO Foo VALUES (1), (2), (3)
    """
        )
        == [{"type": "INSERT", "affected": 3}]
    )

    assert (
        db.query(
            """
    INSERT INTO Bar VALUES
        ('{ "hello": 1 }'),
        ('{ "world": "cookie" }');
    """
        )
        == [{"type": "INSERT", "affected": 2}]
    )

    assert (
        db.query(
            """
    SELECT * FROM Bar
    """
        )
        == [{"type": "SELECT", "rows": [{"hello": 1}, {"world": "cookie"}]}]
    )

    assert (
        db.query(
            """
    SELECT * FROM Bar
            """
        )
        == [{"type": "SELECT", "rows": [{"hello": 1}, {"world": "cookie"}]}]
    )

    assert (
        db.query(
            """
    SELECT * FROM Foo
            """
        )
        == [{"type": "SELECT", "rows": [{"id": 1}, {"id": 2}, {"id": 3}]}]
    )

    assert (
        db.query(
            """
    UPDATE Foo SET id = id + 2 WHERE id = 3
            """
        )
        == [{"type": "UPDATE", "affected": 1}]
    )

    assert (
        db.query(
            """
    DELETE FROM Foo WHERE id < 5
            """
        )
        == [{"type": "DELETE", "affected": 2}]
    )

    assert (
        db.query(
            """
    SELECT * FROM Foo
            """
        )
        == [{"type": "SELECT", "rows": [{"id": 5}]}]
    )

    assert (
        db.query(
            """
    SHOW COLUMNS FROM Foo
            """
        )
        == [{"type": "SHOW COLUMNS", "columns": [{"name": "id", "type": "INT"}]}]
    )

    assert (
        db.query(
            """
    SHOW TABLES
            """
        )
        == [{"type": "SHOW TABLES", "tables": ["Bar", "Foo"]}]
    )

    assert (
        db.query(
            """
    DROP TABLE IF EXISTS Foo
            """
        )
        == [{"affected": 1, "type": "DROP TABLE"}]
    )



"""
Type hints for Native Rust Extension
"""

from abc import ABCMeta
from typing import Any, Final

class Glue:
    def __init__(self, storage: "Storage") -> None: ...
    def query(sql: str): ...

class Storage(metaclass=ABCMeta):
    pass

class MemoryStorage(Storage):
    def __init__(): ...

class SharedMemoryStorage(Storage):
    def __init__(): ...

class JsonStorage(Storage):
    def __init__(path: str): ...

class SledStorageModeConfig:
    """
    In this mode, the database will make
    decisions that favor using less space
    instead of supporting the highest possible
    write throughput. This mode will also
    rewrite data more frequently as it
    strives to reduce fragmentation.
    """
    LowSpace: Final[Any]

    """
    In this mode, the database will try
    to maximize write throughput while
    potentially using more disk space.
    """
    HighThroughput: Final[Any]

class SledStorageConfig:
    def __init__(
        *,
        path: str,
        temporary: bool,
        use_compression: bool,
        print_profile_on_drop: bool,
        compression_factor: int,
        create_new: bool,
        cache_capacity: int,
        mode: "SledStorageModeConfig",
    ): ...
    path: str
    temporary: bool
    use_compression: bool
    print_profile_on_drop: bool
    compression_factor: int
    create_new: bool
    cache_capacity: int
    mode: "SledStorageModeConfig"

class SledStorage(Storage):
    def __init__(path: str): ...
    @staticmethod
    def try_from(config: "SledStorageConfig") -> "SledStorage": ...

class GlueSQLError(Exception):
    pass



from gluesql import Glue, SharedMemoryStorage
from tabulate import tabulate

db = Glue(SharedMemoryStorage())


def run():
    db.query(
        """
        CREATE TABLE User (id INTEGER, name TEXT);
        CREATE TABLE Device (name TEXT, userId INTEGER);
        INSERT INTO User VALUES
        (1, 'glue'), (2, 'sticky'), (3, 'watt');
        INSERT INTO Device VALUES
        ('Phone', 1), ('Mic', 1), ('Monitor', 3),
        ('Mouse', 2), ('Touchpad', 2);
        """
    )

    sql = "SHOW TABLES;"
    result = db.query(sql)
    tables = result[0].get("tables")
    tables = list(map(lambda t: [t], tables))
    print(f"\n[Query]\n{sql}")
    print(
        tabulate(tables, headers=["Values"], showindex=True, tablefmt="simple_outline")
    )

    sql = """
        SELECT
        u.name as user,
        d.name as device
        FROM User u
        JOIN Device d ON u.id = d.userId
    """.strip().replace(
        "    ", ""
    )

    result = db.query(sql)
    rows = result[0].get("rows")
    print(f"\n[Query]\n{sql}")
    print(tabulate(rows, headers="keys", showindex=True, tablefmt="simple_outline"))


if __name__ == "__main__":
    run()
